/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.builder;

import java.util.ArrayList;
import javax.swing.SwingWorker;
import tp.model.DocRepresentTable;
import tp.model.DocsRepresentDB;
import tp.model.FeatureFrequency;
import tp.model.FeatureInterface;

/**
 * Class for building the statistics about the whole dataset of text documents
 * @author Ondrej Sabatka
 */

public class BuilderStatistics extends SwingWorker<Void, Void> {

    /**
     * Constructor of the class
     */
    public BuilderStatistics() {}


    /**
     * The method scans all features of all documents and counts the overall statistics
     * @param database database of document representations
     * @return databse of document representations with statistics
     */
    public DocsRepresentDB calculateStatistics(DocsRepresentDB database) {
        DocRepresentTable document;
        ArrayList<FeatureInterface> doc_features;

        //Computed during the first scan
        int count_features = database.getFeaturesDocumentFreq().size();  //count of all features
        int max_count_features = 0;  //max. count of feature in documents
        double avg_count_features = 0.0;  //avg. count of feature in documents

        //Computed during the second scan
        int max_frequent_feature = 0;  //max. frequency of a feature
        double avg_frequent_feature = 0.0;  //avg. frequency of a feature
        double max_tf_feature = 0.0;
        double avg_tf_feature = 0.0;
        double max_tfidf_feature = 0.0;
        double avg_tfidf_feature = 0.0;


        
        //Scans all documents
        for(int i = 0; i < database.getCountDocuments(); i++) {

            document = database.getDocumentAt(i);
            doc_features = document.getArrayFeaturesFrequency();

            //Gets max. count of feature in documents
            if(doc_features.size() > max_count_features) {
                max_count_features = doc_features.size();
            }

            //avg. count of feature in documents
            avg_count_features = avg_count_features + (float) (doc_features.size() / (float) database.getCountDocuments());

            if(database.getModel() != DocsRepresentDB.RepresentationModel.BINARY) {
                setProgress((i * 100 / database.getCountDocuments()) / 2);
            }
            else {
                setProgress(i * 100 / database.getCountDocuments());
            }
        }




        if(database.getModel() != DocsRepresentDB.RepresentationModel.BINARY) {

            //The second scan...
            for(int i = 0; i < database.getCountDocuments(); i++) {

                document = database.getDocumentAt(i);
                doc_features = document.getArrayFeaturesFrequency();

                //Scanning of all features
                for(int j = 0; j < doc_features.size(); j++) {
                    FeatureFrequency f = (FeatureFrequency) doc_features.get(j);

                    if(f.getRate() > max_frequent_feature) {
                        max_frequent_feature = f.getRate();
                    }

                    if(f.getTF() > max_tf_feature) {
                        max_tf_feature = f.getTF();
                    }

                    if(f.getTFidf() > max_tfidf_feature) {
                        max_tfidf_feature = f.getTFidf();
                    }

                    avg_frequent_feature = avg_frequent_feature +
                            (float) (f.getRate() / (float) count_features);

                    avg_tf_feature = avg_tf_feature +
                            (float) (f.getTF() / (float) count_features);

                    avg_tfidf_feature = avg_tfidf_feature +
                            (float) (f.getTFidf() / (float) count_features);
                }


                setProgress(50 + (i * 100 / database.getCountDocuments()) / 2);
            }
        }


        //Storing of the statistics
        database.setCountFeatures(count_features);
        database.setMaxCountFeatures(max_count_features);
        database.setAvgCountFeatures(avg_count_features);

        database.setMaxFrequentFeature(max_frequent_feature);
        database.setAvgFrequentFeature(avg_frequent_feature);
        database.setMaxTFFeature(max_tf_feature);
        database.setAvgTFFeature(avg_tf_feature);
        database.setMaxTFidfFeature(max_tfidf_feature);
        database.setAvgTFidfFeature(avg_tfidf_feature);


        return database;
    }





    
    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
