/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.builder;

import java.util.ArrayList;
import javax.swing.SwingWorker;
import tp.model.DocRepresentTable;
import tp.model.DocsRepresentDB;
import tp.model.FeatureFrequency;
import tp.model.FeatureInterface;

/**
 * A class, which builds the weight vectors for features representing the text documents
 * (for TF or TF-IDF representation)
 * @author Ondrej Sabatka
 */

public class BuilderVector extends SwingWorker<Void, Void> {

    /**
     * Constructor of the class
     */
    public BuilderVector() {}


    /**
     * Scans all features of all documents and counts their TF values
     * @param database database of documents representations
     * @return document representation with weight vectors
     */
    public DocsRepresentDB calculateTF(DocsRepresentDB database) {
        DocRepresentTable document;
        ArrayList<FeatureInterface> doc_features;
        float tf;

        //Scans all documents
        for(int i = 0; i < database.getCountDocuments(); i++) {

            document = database.getDocumentAt(i);
            doc_features = document.getArrayFeaturesFrequency();

            //Scans all features
            for(int j = 0; j < doc_features.size(); j++) {
                FeatureFrequency f = (FeatureFrequency) doc_features.get(j);

                //TF = occurenceof feature in a document / Count of all features in a document
                tf = (float) (f.getRate() / (float) doc_features.size());

                //TF value is stored
                f.setTF(tf);
            }


            setProgress(i * 100 / database.getCountDocuments());
        }


        return database;
    }


    /**
     * Scans all features of all documents and counts their TF-IDF values
     * @param database database of documents representations
     * @return document representation with weight vectors
     */
    public DocsRepresentDB calculateTFIDF(DocsRepresentDB database) {
        DocRepresentTable document;
        ArrayList<FeatureInterface> doc_features;
        float idf, tfidf, tf;

       //Scans all documents
        for(int i = 0; i < database.getCountDocuments(); i++) {

            document = database.getDocumentAt(i);
            doc_features = document.getArrayFeaturesFrequency();

            //Scans all features
            for(int j = 0; j < doc_features.size(); j++) {
                FeatureFrequency f = (FeatureFrequency) doc_features.get(j);

            //TF = occurenceof feature in a document / Count of all features in a document
                tf = (float) (f.getRate() / (float) doc_features.size());
                idf = (float) (Math.log(database.getCountDocuments() /
                        (float) database.getFeatureFreqInCorpus(f.getFeature())));

                //Counting of TF-IDF
                tfidf = tf * idf;

                //TF-IDF value is stored
                f.setTFidf(tfidf);
            }


            setProgress(i * 100 / database.getCountDocuments());
        }


        return database;
    }

    
    
    
    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
