/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.controllers;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.lang.reflect.Method;
import java.util.EventObject;
import java.util.StringTokenizer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Abstract class to process events by the application controller.
 *
 * Component application controllers inherit properties of this class and process
 * events, which belong to them.
 * @author Ondrej Sabatka
 */

abstract public class ActionControl implements ActionListener, ItemListener, ChangeListener {

    //Class properties
    protected EventObject event;


    /**
     * A method, which is called, if some event happens
     * @param evt object of the event
     */
    public void actionPerformed(ActionEvent evt) {
        this.event = evt;
        //Determination of a method name, which should be executed
        String action = ((Component) (evt.getSource())).getName();

        if(!action.contains(";")) {
            invokeMethod(action);
        }
        else {
            StringTokenizer st = new StringTokenizer(action, ";");
            String method_name = st.nextToken();
            Object[] arguments = null;
            int count_args = 0;
            
            while(st.hasMoreTokens()) {
                arguments = new Object[++count_args];
                arguments[count_args - 1] = new String(st.nextToken());
            }
            
            invokeMethod(method_name, arguments);
        }
    }


    /**
     * A method, which is called, if some checkbox is changed
     * @param evt object of the event
     */
    public void itemStateChanged(ItemEvent evt) {
        this.event = evt;
        //Determination of a method name, which should be executed
        invokeMethod(((Component) (evt.getSource())).getName());
    }


    /**
     * A method, which is called, if a value of some spinner is changed
     * @param evt object of the event
     */
    public void stateChanged(ChangeEvent evt) {
        this.event = evt;
        //Determination of a method name, which should be executed
        invokeMethod(((Component) (evt.getSource())).getName());
    }


    /**
     * Calls a method, which name is given as a parameter
     * @param action name of the method
     */
    public void invokeMethod(String action) {
        //Tries, if the method is implemented
        try {
            Method method = this.getClass().getMethod(action);
            method.invoke(this);
        }
        catch(Exception e) {}
    }


    /**
     * Calls a method, which name is given as a first parameter with parameter(s)
     * given as next parameter(s) of the method
     * @param method_name name of the method
     * @param arguments array of parameters
     */
    private void invokeMethod(String method_name, Object[] arguments) {
        //Array of parameters is created
        Class arg_types[] = new Class[arguments.length];
        for(int i = 0; i < arguments.length; i++) {
            arg_types[i] = String.class;
        }

        //Tries, if the method is implemented
        try {
            Method method = this.getClass().getMethod(method_name, arg_types);
            method.invoke(this, arguments);
        }
        catch(Exception e) {}
    }

}
