/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */
/* metody jsou castecne prevzaty a upraveny z diplomove prace Ing. Martina Uhlire */
package tp.loader;

/**
 * Class for cleaning of loaded text data.
 * @author Ondrej Sabatka
 */

public class DataCleaner {

    //Variables
    static char[] punctuation = { ',', ':', '-' };
    

    //Constructor
    public DataCleaner() {}


    /**
     * Method removes some special symbols, which occur in Reuters dataset.
     * @param str original string
     * @return cleaned string
     */
    public String cleanSGMLReuters(String str) {
        // word with zero size is ignored
        if (str.length() == 0) {
            str = "";
            return str;
        }

        // replacement of HTML tags
        if (str.charAt(0) == '&') {
            str = replaceHTMLtags(str);
        }

        // all chars to lowercase
        str = str.toLowerCase();

        // remove non literals
        str = removeNonLiterals(str);

        return str;
    }

    /**
     * Method removes some special symbols, which occur in topic of Reuters documents.
     * @param str original string
     * @return cleaned string
     */
    public String cleanTopic(String topic) {
        String clean_topic = topic.toLowerCase();
        clean_topic = clean_topic.replaceAll("<[^>]*>", " ");
        clean_topic = clean_topic.trim();
        return clean_topic;
    }

    /**
     * Method removes some special symbols, which occur in contents of Reuters documents.
     * @param str original string
     * @return cleaned string
     */
    public String cleanContentFile(String content) {
        String clean_content;

        clean_content = content.replaceAll("<[^>]*>", "");

        return clean_content;
    }


    /**
     * Method removes some special symbols, which occur in plain text documents.
     * @param str original string
     * @return cleaned string
     */
    public String cleanPlainText(String str) {

        // word with zero size is ignored
        if (str.length() == 0) {
            str = "";
            return str;
        }

        // all chars to lowercase
        str = str.toLowerCase();

        // remove non literals
        str = removeNonLiterals(str);

        return str;
    }


    /**
     * Method transforms HTML notation of special chars to normal characters
     * @param str original string
     * @return result string
     */
    private String replaceHTMLtags(String str) {
        if (str.startsWith("&lt;")) {
            str = str.replaceAll("&lt;", "<");
        }
        else if (str.startsWith("&amp;")) {
            str = str.replaceAll("&amp;", "&");
        }
        else {
            str = str.replaceAll("&#[0-9]+;", "");
        }

        return str;
    }


    /**
     * Method removes puntuation marks
     *
     * @param str original string
     * @return result string
     */
    private String removeNonLiterals(String str) {
        if (str.length() > 0) {
            if(isNumber(str)) {
                return "";
            }

            // special case "u.s."
            if (str.equals("u.s.")) {
                return str;
            }

            // 1st character of a string
            char char1 = str.charAt(0);

            switch (char1) {
                // these characters are removed
                case '\"':
                case '(':
                case '\'':
                case '-':
                    str = str.substring(1, str.length());
                    str = removeNonLiterals(str);
                    break;

            }

            // last character of a string
            if (str.length() == 0) {
                return "";
            }

            char1 = str.charAt(str.length() - 1);

            switch (char1) {
                // these characters are removed (at the end)
                case '\"':
                case ')':
                case '\'':
                case '-':
                    str = str.substring(0, str.length() - 1);
                    str = removeNonLiterals(str);
                    break;
            }

            // punctuation mark at the end of word
            if (str.length() == 0) {
                return "";
            }


            for (int i = 0; i < punctuation.length; i++) {
                if (str.charAt(str.length() - 1) == punctuation[i]) {
                    str = str.substring(0, str.length() - 1);
                    str = removeNonLiterals(str);
                    break;
                }
            }
        }

        str = str.replaceAll("-", " ");
        str = str.replaceAll("%", " ");

        //Removes words beginning with a number
        str = str.replaceAll("[0-9]+[^ ]*", "");

        //Removes words beginning with some other special characters
        str = str.replaceAll("'[^ ]*", "");


        return str;
    }


    /**
     * Tests, if a word can be converted to number
     * @param string input string
     * @return true, if it is a number, false otherwise
     */
    private boolean isNumber(String str) {
        if(str.matches("[0-9]+") ||
           str.matches("[0-9]+.[0-9]*") ||
           str.matches("[0-9]+,[0-9,]*") ||
           str.matches("[0-9]+.[0-9]+e[0-9]+")) {
            return true;
        }

        return false;
    }

}
