/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.loader;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;
import javax.swing.SwingWorker;
import tp.controllers.MainControl;
import tp.model.Document;
import tp.model.DocumentsDatabase;
import tp.model.LinkToFiles;


/**
 * Class for loading the plain text files.
 * @author Ondrej Sabatka
 */

public class LoaderPlainText extends SwingWorker<Void, Void> implements LoaderInterface {

    //Constructor
    public LoaderPlainText() {}


    /**
     * Method for loading plain text files - one document for each file.
     * Reads the files given by the user and returns teh database of documents.
     * @param link_files links to files for loading
     * @return reference to the database of documents
     */
    public DocumentsDatabase loadFromFiles(LinkToFiles link_files) {
        DocumentsDatabase database = new DocumentsDatabase();
        DataCleaner cleaner = new DataCleaner();
        String content_file = null;
        String topic = null;

        //Sacnning of all text files
        for(int i = 0; i < link_files.getCount(); i++) {
            //Initialization of a string for one file contents
            content_file = new String();

            //Read the file...
            content_file = loadFile(link_files.getFileAt(i));
            topic = loadFileTopic(link_files.getFileAt(i));

            if(content_file == null) {
                return null;
            }

            //Cleaning of document contents
            content_file = cleaner.cleanContentFile(content_file);

            //Adding a document into database
            database.addDocument(new Document(content_file, topic));

            setProgress(i * 100 / link_files.getCount());
        }

        
        return database;
    }


    /**
     * Takes one file and makes a string from ites contents, which is returned.
     * Redundant white characters are removed and all words are changed to lowercase.
     * @param f plain text file
     * @return string with file contents
     */
    public String loadFile(File f) {
        BufferedReader fin = null;
        String line, content_file = new String();
        DataCleaner cleaner = new DataCleaner();

        try {
            //Input stream for reading
            fin = new BufferedReader(new FileReader(f.getAbsoluteFile()));

            //File contents are stored into content_file string
            while ((line = fin.readLine()) != null) {
                //Scans all words in a row and removes white chars
                StringTokenizer st = new StringTokenizer(line);

                while (st.hasMoreTokens()) {
                    String word = st.nextToken();

                    //Each word is cleaned and stored
                    content_file = content_file + " " + cleaner.cleanPlainText(word);
                }

                content_file = content_file.trim();
            }
            
        } catch (FileNotFoundException e) {
            MainControl.getInstance().getMainView().
                        showErrorDialog("File not found!");
            return null;
        }
        catch (IOException e) {
            System.out.println (e.getMessage());
            MainControl.getInstance().getMainView().
                        showErrorDialog("Error in file reading!");
            return null;
        }
        
        return content_file;
    }


    /**
     * Returns the class of a document; it is taken from name of a parent directory
     * @param fileAt a text file
     * @return class of the document in a file
     */
    private String loadFileTopic(File fileAt) {
        String topic = fileAt.getParent();
        String s = "";
     try {
        File topic_dir = fileAt.getParentFile();
        String redundant = topic_dir.getParent() + File.separator;
        int ind = topic.lastIndexOf(File.separatorChar);
        s = topic.substring(ind+1);
//        s = topic.replaceAll(redundant, "");
       } catch(Exception e) {
         System.out.println (e.getMessage());
     }
        return s; 
    }


    @Override
    protected Void doInBackground() throws Exception {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
