/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.model;

import java.util.Vector;

/**
 * This class serves as a transactional database of textual documents.
 * It allows to store texts of documents and work with them.
 * @author Ondrej Sabatka
 */

public class DocumentsDatabase {

    //Variables
    private Vector<Document> document_base;

    //Cursor
    private int cursor;
    private static final int NOT_INIT = -1;
    private static final int START = 0;


    /**
     * Constructor of the class.
     */
    public DocumentsDatabase() {
        document_base = new Vector<Document>();
        cursor = NOT_INIT;
    }


    /**
     * Adds a document to the database.
     * @param document document to add
     */
    public void addDocument(Document document) {
        document.setID(document_base.size());
        document_base.add(document);
    }


    /**
     * Method stores a document to a given position.
     * @param index position
     * @param document document
     */
    public void setDocoumentTo(int index, Document document) {
        document_base.set(index, document);
    }


    /**
     * Method returns a document being at position next to the cursor and
     * increments value of the cursor.
     * @return one document
     */
    public Document getDocumentSingly() {
        //First usage of cursor
        if(cursor <= NOT_INIT) {
            cursor = START; //1st position
            return document_base.get(cursor++);
        }
        //If it is not at the end of database, returns the next document
        else if(cursor < document_base.capacity()) {
            return document_base.get(cursor++);
        }
        else {
            cursor = NOT_INIT;
            return null; //returns NULL (end of database)
        }
    }


    /**
     * Method returns a document being at a given position.
     * @param index position of a document
     * @return a document
     */
    public Document getDocumentAt(int index) {
        return document_base.get(index);
    }

    
    /**
     * Returns the size of database (count of documents)
     * @return count of documents
     */
    public int getCountDocuments() {
        return document_base.size();
    }

}
