/**
 * Prakticka cast diplomove prace
 * Reprezentace textu a její vliv na kategorizaci
 *
 * Projekt: Program pro predzpracovani textovych dat
 *            - nacita textova data nebo dataset Reuters-21578
 *            - umoznuje nastavit metody predzpracovani, vybrat model
 *              a rysy reprezentace
 *            - zameruje se take na reprezentaci dokumentu pomoci N-Gramu
 *            - uklada data ve formatu XML nebo CSV
 * Autor:   Bc. Ondrej Sabatka
 * Kontakt: xsabat05@stud.fit.vutbr.cz
 * Datum:   unor - duben 2010
 */

package tp.view;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import tp.controllers.MainControl;

/**
 * Class, which is displaying the panel for data loading in the application.
 * @author Ondrej Sabatka
 */

public class TabLoaderView extends JPanel {

    //Variables
    private static String[] documentsFormat = {"Plain text", "Reuters-21578"};

    //Possibilities of document format
    public static final int PLAIN_TEXT = 0;
    public static final int REUTERS = 1;

    private static final int WIDTH_FILE_LIST = 400;
    private static final int HEIGHT_FILE_LIST = 220;
    private final JFileChooser fc = new JFileChooser();
    private ActionListener action_handler;
    private JList file_list;
    private JComboBox select_format;


/**
 * Constructor of the class.
 */
    public TabLoaderView() {
        //Controller setting
        action_handler = MainControl.getInstance().getLoaderControl();

        //Layout setting
        GridBagConstraints area = new GridBagConstraints();
        this.setLayout(new GridBagLayout());
        area.fill = GridBagConstraints.BOTH;

        //Fonts class...
        GUIFonts font = new GUIFonts();


        ////////////////////////////////////////////////////////////////////////
        //                        FORMAT SETTING PANEL                        //
        ////////////////////////////////////////////////////////////////////////
        JPanel format_panel = new JPanel();
        format_panel.setBorder(BorderFactory.createTitledBorder("File formats"));

        //Help about format setting...
        format_panel.setName("load_format.html");
        format_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridx = 0;
        area.gridy = 0;
        area.weightx = 0.5;
        area.weighty = 0.05;

        this.add(format_panel, area);


        //Fomat setting panel components
        JLabel label = new JLabel("Choose input data format: ");
        label.setFont(font.getText());
        format_panel.add(label);
        //ComboBox to choose the format
        select_format = new JComboBox(documentsFormat);
        select_format.setFont(font.getBoldText());
        format_panel.add(select_format);


        ////////////////////////////////////////////////////////////////////////
        //                       DOCUMENT LOADING PANEL                       //
        ////////////////////////////////////////////////////////////////////////
        JPanel load_panel = new JPanel();
        load_panel.setBorder(BorderFactory.createTitledBorder("Select files"));

        //Help about format setting...
        load_panel.setName("load_loading.html");
        load_panel.addMouseListener(MainControl.getInstance().getHelpControl());

        area.gridy = 1;
        area.weighty = 0.9;
        area.fill = GridBagConstraints.BOTH;

        this.add(load_panel, area);


        //Components at the panel of Loaded documents
        label = new JLabel("List of input files:");
        label.setFont(font.getText());
        
        file_list = new JList();

        JScrollPane file_list_scroller = new JScrollPane(file_list);
        file_list_scroller.setPreferredSize(new Dimension(WIDTH_FILE_LIST, HEIGHT_FILE_LIST));

        //Panel of buttons
        JPanel btn_panel = new JPanel();
        JButton load_btn = new JButton("Read files");
        load_btn.setName("loadBtnClick");
        load_btn.addActionListener(action_handler);
        btn_panel.add(load_btn);

        //Button, which deletes chosen files from the list
        JButton delete_btn = new JButton("Remove selected");
        delete_btn.setName("deleteBtnClick");
        delete_btn.addActionListener(action_handler);
        btn_panel.add(delete_btn);

        //Button, which clears the whole list of files
        JButton clear_all_btn = new JButton("Remove all");
        clear_all_btn.setName("clearBtnClick");
        clear_all_btn.addActionListener(action_handler);
        btn_panel.add(clear_all_btn);

        //Iserts components into the panel
        load_panel.add(label);
        load_panel.add(file_list_scroller);
        load_panel.add(btn_panel);
        



        ////////////////////////////////////////////////////////////////////////
        //                            "NEXT" BUTTON                           //
        ////////////////////////////////////////////////////////////////////////
        ImageIcon icon = MainControl.getInstance().getResourceModel().createImageIcon("next.png");
        JButton next_btn = new JButton("Next ", icon);
        next_btn.setFont(font.getTitleText());

        next_btn.setName("nextBtnClick");
        next_btn.addActionListener(action_handler);

        next_btn.setVerticalTextPosition(AbstractButton.CENTER);
        next_btn.setHorizontalTextPosition(AbstractButton.LEADING);

        area.gridy = 2;
        area.weighty = 0.05;
        area.fill = GridBagConstraints.NONE;

        this.add(next_btn, area);
    }



    /**
     * Function, which opens the "Open file" dialog and returns array of marked 
     * files or null, if the dialog is canceled.
     * @return array of chosen files
     */
    public File[] getFilesFromUser() {
        //Setting of "Open" dialog
        fc.setMultiSelectionEnabled(true);

         //Opening of "Open" dialog
        int returnVal = fc.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            return fc.getSelectedFiles();
        } else {
            return null;
        }
    }


    /**
     * Displays the list of files into the field.
     * @param files list of files
     */
    public void fillFileList(String[] files) {
        file_list.setListData(files);
    }


    /**
     * Clears the list of files.
     */
    public void clearFileList() {
        Object[] clear = new Object[0];
        file_list.setListData(clear);
    }


    /**
     * Returs array of all items chosen by the user in the field of files.
     * @return array of chosen items
     */
    public int[] getSelectedFiles() {
        return file_list.getSelectedIndices();
    }


    /**
     * Returns the chosen format of input documents
     * @return format of input documents
     */
    public int getFileFormat() {
        return select_format.getSelectedIndex();
    }

}
